from aiohttp import ClientSession

from typing import Optional


async def request(url: str, data: Optional[dict] = None) -> dict:
	async with ClientSession() as session:
		async with session.get(url=url, params=data) as response:
			return await response.json(encoding="UTF-8")



from typing import Optional

from pydantic import BaseModel

from aioowm.types.city import CityModel
from aioowm.types.weather import WeatherModel


class Model(BaseModel):
    error: Optional[bool] = None
    code: Optional[int] = None
    default: Optional[dict] = None
    city: Optional[CityModel] = None
    weather: Optional[WeatherModel] = None





from typing import Optional


async def temperature_handler(temperature: Optional[str]) -> str:
    """
    :param temperature: accepts temp identifier (celsius, fahrenheit, kelvin)
    :return: units
    """
    identifiers = {"celsius": "metric", "fahrenheit": "imperial", "kelvin": "kelvin"}

    if temperature.lower() not in identifiers:
        raise TypeError("Unknown temperature identifier!")
    else:
        return identifiers[temperature]


__version__ = "2.0.1"
__author__ = "vladislavkovalskyi"

WEATHER_API = "http://api.openweathermap.org/data/2.5/weather?"

WEATHER_CODES = {
    # Гроза
    200: "Гроза с небольшим дождём",
    201: "Гроза с дождём",
    202: "Гроза с сильным дождём",
    210: "Легкая гроза",
    211: "Гроза",
    212: "Сильная гроза",
    221: "Рваная гроза",
    230: "Гроза с мелким дождиком",
    231: "Гроза с моросью",
    232: "Гроза с сильным моросящим дождём",

    # Морось
    300: "Слабая морось",
    301: "Морось",
    302: "Сильный дождь",
    310: "Интенсивный свет и моросящий дождь",
    311: "Моросящий дождь",
    312: "Сильный моросящий дождь",
    313: "Ливень, дождь и изморось",
    314: "Сильный ливень, дождь и изморось",
    321: "Изморось",

    # Дождь
    500: "Легкий дождь",
    501: "Умеренный дождь",
    502: "Сильный дождь",
    503: "Очень сильный дождь",
    504: "Жёсткий дождь",
    511: "Холодный дождь",
    520: "Интенсивный свет и ливень",
    521: "Дождь как в душе",
    522: "Сильный ливневый дождь",
    531: "Рваный душ под дождем",

    # Снег
    600: "Легкий снег",
    601: "Снег",
    602: "Снегопад",
    611: "Мокрый снег",
    612: "Легкий дождь с мокрым снегом",
    613: "Мокрый снег",
    615: "Небольшой дождь и снег",
    616: "Дождь и снег",
    620: "Легкий снегопад",
    621: "Сильный снегопад",
    622: "Очень сильный снегопад",

    # Атмосфера
    701: "Утренний туман",
    711: "Дым",
    721: "Водный туман",
    731: "Песчано-пыльные вихри",
    741: "Ясный туман",
    751: "Песок",
    761: "Пыль",
    762: "Вулканический пепел",
    771: "Шквалы",
    781: "Торнадо",

    # Небо, облака
    800: "Чистое небо",
    801: "Очень мало облаков",
    802: "Мало облаков",
    803: "Облачно",
    804: "Сильно облачно",
}


from typing import Optional

from pydantic import BaseModel


class PressureModel(BaseModel):
    sea_level: Optional[int] = None
    ground_level: Optional[int] = None
    value: Optional[int] = None


class WindModel(BaseModel):
    speed: Optional[float] = None
    direction: Optional[int] = None
    gust: Optional[float] = None


class TemperatureModel(BaseModel):
    now: Optional[float] = None
    minimal: Optional[float] = None
    maximal: Optional[float] = None
    feels_like: Optional[float] = None


class WeatherModel(BaseModel):
    id: Optional[int] = None
    main: Optional[str] = None
    description: Optional[str] = None
    clouds: Optional[int] = None
    rain: Optional[float] = None
    snow: Optional[float] = None
    humidity: Optional[int] = None
    pressure: Optional[PressureModel] = None
    wind: Optional[WindModel] = None
    temperature: Optional[TemperatureModel] = None



from typing import Optional

from aioowm.const import const
from aioowm.handlers.language import language_handler
from aioowm.handlers.response import response_handler
from aioowm.handlers.temperature import temperature_handler
from aioowm.http.http import request
from aioowm.types.model import Model


class OWM:
    def __init__(
            self,
            token: str,
            language: Optional[str] = "en",
            temperature: Optional[str] = "celsius"
    ):
        """
        :param token: your api-key https://home.openweathermap.org/api_keys
        :param language: accepts country identifier (en, ru, fr)
        :param temperature: accepts temperature (celsius, fahrenheit, kelvin)
        """

        self._token = token
        self.language = language
        self.temperature = temperature

    async def get(self, city: str) -> Model:
        response = await request(
            const.WEATHER_API, {
                "q": city,
                "lang": await language_handler(self.language),
                "units": await temperature_handler(self.temperature),
                "appid": self._token
            }
        )
        response = await response_handler(response)
        return Model(**response)



from typing import Optional

from pydantic import BaseModel


class CoordinatesModel(BaseModel):
    longitude: Optional[float] = None
    latitude: Optional[float] = None


class CityModel(BaseModel):
    id: Optional[int] = None
    name: Optional[str] = None
    country: Optional[str] = None
    timezone: Optional[int] = None
    sunrise: Optional[int] = None
    sunset: Optional[int] = None
    coordinates: Optional[CoordinatesModel] = None



from typing import Optional

from aioowm.const.const import WEATHER_CODES


async def response_handler(response: Optional[dict]) -> dict:
    if response["cod"] != 200:
        return {"error": True, "code": response["cod"]}
    return {
        "error": False,
        "code": response["cod"],
        "default": response,
        "city": {
            "id": response["id"],
            "name": response["name"],
            "country": response["sys"]["country"],
            "timezone": response["timezone"],
            "sunrise": response["sys"]["sunrise"],
            "sunset": response["sys"]["sunset"],
            "coordinates": {
                "longitude": response["coord"]["lon"],
                "latitude": response["coord"]["lat"]
            }
        },
        "weather": {
            "id": response["weather"][0]["id"],
            "main": response["weather"][0]["main"],
            "description": WEATHER_CODES[response["weather"][0]["id"]],
            "clouds": response["clouds"]["all"],
            "rain": response["rain"].get("1h") if "rain" in response else None,
            "snow": response["snow"].get("1h") if "snow" in response else None,
            "humidity": response["main"].get("humidity"),
            "pressure": {
                "sea_level": response["main"].get("sea_level"),
                "ground_level": response["main"].get("grnd_leve,"),
                "value": response["main"].get("pressure")
            },
            "wind": {
                "speed": response["wind"].get("speed"),
                "direction": response["wind"].get("deg"),
                "gust": response["wind"].get("gust")
            },
            "temperature": {
                "now": response["main"].get("temp"),
                "minimal": response["main"].get("temp_min"),
                "maximal": response["main"].get("temp_max"),
                "feels_like": response["main"].get("feels_like")
            }
        }
    }



# python setup.py sdist bdist_wheel
# python -m pip install --upgrade twine
# python -m twine upload dist/*
import setuptools

try:
    with open("README.md", "r", encoding="utf-8") as f:
        long_description = f.read()
except UnicodeDecodeError:
    with open("README.md", "r", encoding="utf-8") as f:
        long_description = f.read()
except FileNotFoundError:
    long_description = ""

with open("aioowm/const/const.py", encoding="utf-8") as f:
    exec(f.read())

setuptools.setup(
    name="aioowm",
    version=locals()["__version__"],
    author=locals()["__author__"],
    description="Async OpenWeatherMap API!",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    url="https://github.com/vladislavkovalskyi/aioowm",
    packages=setuptools.find_packages(),
    author_email="v.darknesssb@gmail.com",
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Operating System :: OS Independent",
    ],
    install_requires=["aiohttp", "pydantic"]
)



from typing import Optional


async def language_handler(language: Optional[str] = "ru") -> str:
    """
    :param language: accepts language identifier (ru, en, fr)
    :return: True if lang in identifiers. False if lang not in identifiers
    """

    identifiers = ("af", "al", "ar", "az", "bg", "ca", "cz", "da", "de", "el", "en", "eu",
                   "fa", "fi", "fr", "gl", "he", "hi", "hr", "hu", "id", "it", "ja", "kr",
                   "la", "lt", "mk", "no", "nl", "pl", "pt", "pt_br", "ro", "ru", "sv", "se",
                   "sk", "sl", "sp", "es", "sr", "th", "tr", "ua", "uk", "vi", "zh_cn", "zn_tw",
                   "zu")

    if language.lower() not in identifiers:
        raise TypeError("Unknown language identifier!")
    else:
        return language.lower()